/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/



#include "GLFFile.h"
#include "GeoLib.h"
#include "ProgressUpdater.h"



class WithComma: public numpunct<char> // class for decimal numbers with comma
{
	protected: char do_decimal_point() const { return ','; } // override the function that gives the decimal separator
};

locale myloc(locale(), new WithComma);// Own numeric facet


bool CGLFFile::m_sbFormat2 = false;





/**--------------------------------------------------------------------------<BR>
CGLFFile::SetArcFormat
\brief See header for format 2 description.
<P>---------------------------------------------------------------------------*/
void CGLFFile::SetFormat(bool bFormat2)
{
	m_sbFormat2 = bFormat2;
}


/**--------------------------------------------------------------------------<BR>
CGLFFile::SetArcFormat
\brief See header for format 2 description.
<P>---------------------------------------------------------------------------*/
bool CGLFFile::GetFormat(void)
{
	return m_sbFormat2;
}


/**--------------------------------------------------------------------------<BR>
CGLFFile::InitSave
\brief InitSave.
<P>---------------------------------------------------------------------------*/
bool CGLFFile::InitSave(fstream& sw, const char* strFile, bool bAppend)
{
	if (bAppend)
		sw.open (strFile, fstream::out | fstream::app);
	else
		sw.open (strFile, fstream::out);

	if (sw.fail())
	{
		sw.close();

		return false;
	}

	sw.setf(ios::fixed,ios::floatfield);

	sw.precision(10);

	return true;
}


/**--------------------------------------------------------------------------<BR>
CGLFFile::InitLoad
\brief InitLoad.
<P>---------------------------------------------------------------------------*/
bool CGLFFile::InitLoad(fstream& sw, const char* strFile)
{
	sw.open (strFile, fstream::in);

	if (sw.fail())
	{
		sw.close();

		return false;
	}

	sw.setf(ios::fixed,ios::floatfield);

	sw.precision(10);

	return true;
}


/**--------------------------------------------------------------------------<BR>
CGLFFile::Save
\brief Save.
<P>---------------------------------------------------------------------------*/
bool CGLFFile::Save(const char* strFile, const C2DPolyBase& rPoly, bool bAppend)
{
	std::fstream filestr;

	if (!InitSave(filestr, strFile, bAppend))
		return false;

	filestr << "(+)\n";

	Store(filestr, rPoly);

	filestr.close();

	return true;

}


/**--------------------------------------------------------------------------<BR>
CGLFFile::Save
\brief Save.
<P>---------------------------------------------------------------------------*/
bool CGLFFile::Save(const char* strFile, const C2DBaseSet& rPoly, bool bAppend, CProgressUpdater* pUpdater)
{
	std::fstream filestr;

	if (!InitSave(filestr, strFile, bAppend))
		return false;

	unsigned int nSize = rPoly.size();

	for (unsigned int i = 0; i < nSize; i++)
	{
		const C2DBase* pBase = rPoly.GetAt(i);
		
		C2DBase::E_TYPE eType = pBase->GetType();

		if (eType == C2DBase::PolyBase || eType == C2DBase::PolyArc || eType == C2DBase::PolyLine)
		{
			Store(filestr, *dynamic_cast<const C2DPolyBase*>(pBase));
		}
		else if (eType == C2DBase::PolyArcHoled || eType == C2DBase::PolyHoledBase || eType == C2DBase::PolyLineHoled)
		{
			Store(filestr, *dynamic_cast<const C2DHoledPolyBase*>(pBase));
		}

		if (pUpdater)
			pUpdater->UpdateProgress(i * 100 / nSize);
	}

	filestr.close();

	return true;
}


/**--------------------------------------------------------------------------<BR>
CGLFFile::LoadPolygon
\brief LoadPolygon.
<P>---------------------------------------------------------------------------*/
C2DPolygon* CGLFFile::LoadPolygon(const char* strFile)
{
	C2DPolygon* pResult = 0;

	std::fstream filestr;

	if (InitLoad(filestr, strFile))
	{
		pResult = RestorePolygon(filestr);

		filestr.close();
	}

	return pResult;
}


/**--------------------------------------------------------------------------<BR>
CGLFFile::LoadHoledPolygon
\brief LoadHoledPolygon.
<P>---------------------------------------------------------------------------*/
C2DHoledPolygon* CGLFFile::LoadHoledPolygon(const char* strFile)
{
	C2DHoledPolygon* pResult = NULL;

	std::fstream filestr;

	if (InitLoad(filestr, strFile))
	{
		C2DPolygon* pPoly = RestorePolygon(filestr);

		if (pPoly)
		{
			pResult = new C2DHoledPolygon();
			pResult->SetRimDirect(pPoly);

			bool bHole;
			while ((pPoly = RestorePolygon(filestr, &bHole)) != 0 && bHole)
			{
				pResult->AddHoleDirect(pPoly);
			}

			if (pPoly)
				delete pPoly;
		}

		filestr.close();
	}

	return pResult;
}


/**--------------------------------------------------------------------------<BR>
CGLFFile::LoadPolyArc
\brief LoadPolyArc.
<P>---------------------------------------------------------------------------*/
C2DPolyArc* CGLFFile::LoadPolyArc(const char* strFile)
{
	C2DPolyArc* pResult = 0;

	std::fstream filestr;

	if (InitLoad(filestr, strFile))
	{
		C2DPolyArc* pResult = RestorePolyArc(filestr);

		filestr.close();
	}

	return pResult;
}


/**--------------------------------------------------------------------------<BR>
CGLFFile::LoadHoledPolyArc
\brief LoadHoledPolyArc.
<P>---------------------------------------------------------------------------*/
C2DHoledPolyArc* CGLFFile::LoadHoledPolyArc(const char* strFile)
{
	C2DHoledPolyArc* pResult = NULL;

	std::fstream filestr;

	if (InitLoad(filestr, strFile))
	{
		C2DPolyArc* pPoly = RestorePolyArc(filestr);

		if (pPoly)
		{
			pResult = new C2DHoledPolyArc();
			pResult->SetRimDirect(pPoly);

			bool bHole;
			while ((pPoly = RestorePolyArc(filestr, &bHole)) != 0 && bHole)
			{
				pResult->AddHoleDirect(pPoly);
			}

			if (pPoly)
				delete pPoly;
		}

		filestr.close();
	}
	return pResult;
}

/**--------------------------------------------------------------------------<BR>
CGLFFile::GetFileLength
\brief GetFileLength.
<P>---------------------------------------------------------------------------*/
unsigned int CGLFFile::GetFileLength(fstream& sw)
{
	sw.seekg (0, ios::end);

	unsigned int nlength = sw.tellg();

	sw.seekg (0, ios::beg);

	return nlength;
}


/**--------------------------------------------------------------------------<BR>
CGLFFile::Load
\brief Load.
<P>---------------------------------------------------------------------------*/
bool CGLFFile::Load(const char* strFile, C2DPolygonSet& polys, CProgressUpdater* pUpdater)
{
	std::fstream filestr;

	if (InitLoad(filestr, strFile))
	{
		unsigned int nLength = GetFileLength(filestr);

		C2DPolygon* pPoly = 0;

		while ((pPoly = RestorePolygon(filestr)) != 0)
		{
			polys.Add(pPoly);

			if (pUpdater)
				pUpdater->UpdateProgress(filestr.tellg() * 100 / nLength);
		}

		filestr.close();

		return true;
	}
	else
	{
		return false;
	}
}

/**--------------------------------------------------------------------------<BR>
CGLFFile::Load
\brief Load.
<P>---------------------------------------------------------------------------*/
bool CGLFFile::Load(const char* strFile, C2DHoledPolygonSet& polys, CProgressUpdater* pUpdater)
{
	std::fstream filestr;

	if (InitLoad(filestr, strFile))
	{
		unsigned int nLength = GetFileLength(filestr);

		C2DHoledPolygon* pHoledPoly = 0;
		C2DPolygon* pPoly = 0;
		bool bHole;

		while ((pPoly = RestorePolygon(filestr, &bHole)) != 0)
		{
			if (bHole)
			{
				if (pHoledPoly == 0)
				{
					delete pPoly;
					pPoly = 0;
				}
				else
				{
					pHoledPoly->AddHoleDirect(pPoly);
				}
			}
			else
			{
				pHoledPoly = new C2DHoledPolygon;
				pHoledPoly->SetRimDirect(pPoly);
				polys.Add(pHoledPoly);
			}


			if (pUpdater)
				pUpdater->UpdateProgress(filestr.tellg() * 100 / nLength);
		}

		filestr.close();

		return true;
	}
	else
	{
		return false;
	}

}

/**--------------------------------------------------------------------------<BR>
CGLFFile::Load
\brief Load.
<P>---------------------------------------------------------------------------*/
bool CGLFFile::Load(const char* strFile, C2DPolyArcSet& polys, CProgressUpdater* pUpdater)
{
	std::fstream filestr;

	if (InitLoad(filestr, strFile))
	{
		unsigned int nLength = GetFileLength(filestr);

		C2DPolyArc* pPoly = 0;

		while ((pPoly = RestorePolyArc(filestr)) != 0)
		{
			polys.Add(pPoly);

			if (pUpdater)
				pUpdater->UpdateProgress(filestr.tellg() * 100 / nLength);
		}

		filestr.close();

		return true;
	}
	else
	{
		return false;
	}

}


/**--------------------------------------------------------------------------<BR>
CGLFFile::Load
\brief Load.
<P>---------------------------------------------------------------------------*/
bool CGLFFile::Load(const char* strFile, C2DHoledPolyArcSet& polys, CProgressUpdater* pUpdater)
{
	std::fstream filestr;

	if (InitLoad(filestr, strFile))
	{
		unsigned int nLength = GetFileLength(filestr);

		C2DHoledPolyArc* pHoledPoly = 0;
		C2DPolyArc* pPoly = 0;
		bool bHole;

		while ((pPoly = RestorePolyArc(filestr, &bHole)) != 0)
		{
			if (bHole)
			{
				if (pHoledPoly == 0)
				{
					delete pPoly;
					pPoly = 0;
				}
				else
				{
					pHoledPoly->AddHoleDirect(pPoly);
				}
			}
			else
			{
				pHoledPoly = new C2DHoledPolyArc;
				pHoledPoly->SetRimDirect(pPoly);
				polys.Add(pHoledPoly);
			}


			if (pUpdater)
				pUpdater->UpdateProgress(filestr.tellg() * 100 / nLength);
		}

		filestr.close();

		return true;
	}
	else
	{
		return false;
	}
}

/**--------------------------------------------------------------------------<BR>
CGLFFile::Load
\brief Loads into a set. Arced polys created when necessary.
<P>---------------------------------------------------------------------------*/
bool CGLFFile::Load(const char* strFile, C2DBaseSet& polybases, CProgressUpdater* pUpdater)
{
	std::fstream filestr;

	C2DHoledPolyArcSet polys;
		
	if (InitLoad(filestr, strFile))
	{
		unsigned int nLength = GetFileLength(filestr);

		C2DHoledPolyArc* pHoledPoly = 0;
		C2DPolyArc* pPoly = 0;
		bool bHole;

		while ((pPoly = RestorePolyArc(filestr, &bHole)) != 0)
		{
			if (bHole)
			{
				if (pHoledPoly == 0)
				{
					delete pPoly;
					pPoly = 0;
				}
				else
				{
					pHoledPoly->AddHoleDirect(pPoly);
				}
			}
			else
			{
				pHoledPoly = new C2DHoledPolyArc;
				pHoledPoly->SetRimDirect(pPoly);
				polys.Add(pHoledPoly);
			}

			if (pUpdater)
				pUpdater->UpdateProgress(filestr.tellg() * 100 / nLength);
		}

		filestr.close();

		while (polys.size() > 0)
		{
			C2DHoledPolyArc* pPoly = polys.ExtractAt(0);
			if (pPoly->HasArcs())
			{
				polybases.Add(pPoly);
			}
			else
			{
				polybases.Add(new C2DHoledPolygon(*pPoly));
				delete pPoly;
			}
		}


		return true;
	}
	else
	{
		return false;
	}
}
	



/**--------------------------------------------------------------------------<BR>
CGLFFile::Save
\brief Save.
<P>---------------------------------------------------------------------------*/
bool CGLFFile::Save(const char* strFile, const C2DHoledPolyBase& rPoly, bool bAppend)
{
	std::fstream filestr;

	if (!InitSave(filestr, strFile, bAppend))
		return false;

	Store(filestr, rPoly);

	filestr.close();

	return true;

}

/**--------------------------------------------------------------------------<BR>
CGLFFile::Store
\brief Store.
<P>---------------------------------------------------------------------------*/
bool CGLFFile::Store(fstream& sw, const C2DHoledPolyBase& rPoly)
{
	const C2DPolyBase* pRim = rPoly.GetRim();

	if (pRim)
	{
		sw << "(+)\n";

		Store(sw, *pRim);

		for (unsigned int i = 0; i < rPoly.GetHoleCount(); i++)
		{
			sw << "(-)\n";
			Store(sw, *rPoly.GetHole(i));
		}
	}
	else
	{
		return false;
	}

	return false;
}





void CGLFFile::StoreFormat1(fstream& sw, const C2DPolyBase& pol)
{
	for (unsigned int i = 0; i < pol.GetLineCount(); i++)
    {
		const C2DLineBase* pLineBase = pol.GetLine(i);

		C2DPoint p = pLineBase->GetPointFrom();

        sw << p.x;
		sw << "\t";
        sw << p.y;
 

		if (pLineBase->GetType() == C2DBase::ArcedLine)
		{
			sw << "\t";

			const C2DArc* pArc = dynamic_cast<const C2DArc*>(pLineBase);

			sw << pArc->GetRadius();

			sw << "\t";

			sw << pArc->GetCentreOnRight();

			sw << "\t";

			sw << pArc->GetArcOnRight();
		}

       sw << "\n";
    }

    sw << "\n";
}


/**--------------------------------------------------------------------------<BR>
CGLFFile::Store
\brief Store.
<P>---------------------------------------------------------------------------*/
void CGLFFile::Store(fstream& sw, const C2DPolyBase& pol)
{
	if (m_sbFormat2)
		StoreFormat2(sw, pol);
	else
		StoreFormat1(sw, pol);
}


/**--------------------------------------------------------------------------<BR>
CGLFFile::Store
\brief Store.
<P>---------------------------------------------------------------------------*/
void CGLFFile::StoreFormat2(fstream& sw, const C2DPolyBase& pol)
{
	unsigned int nCount = pol.GetLineCount();

	if (nCount == 0)
		return;

	C2DPoint p = pol.GetLine(0)->GetPointFrom();
	
    sw << p.x;
	sw << "\t";
    sw << p.y;
    sw << "\n";

	for (unsigned int i = 0; i < nCount; i++)
    {
		const C2DLineBase* pLineBase = pol.GetLine(i);

		C2DPoint p = pLineBase->GetPointTo();

        sw << p.x;
		sw << "\t";
        sw << p.y;

		if (pLineBase->GetType() == C2DBase::ArcedLine)
		{
			sw << "\t";

			const C2DArc* pArc = dynamic_cast<const C2DArc*>(pLineBase);

			sw << pArc->GetRadius();

			sw << "\t";

			sw << pArc->GetCentreOnRight();

			sw << "\t";

			sw << pArc->GetArcOnRight();
		}

       sw << "\n";
    }

    sw << "\n";
}


/**--------------------------------------------------------------------------<BR>
CGLFFile::RestorePolyArc
\brief RestorePolyArc.
<P>---------------------------------------------------------------------------*/
C2DPolyArc* CGLFFile::RestorePolyArc(fstream& sw, bool* bHole)
{
	if (m_sbFormat2)
		return RestorePolyArcFormat2(sw, bHole);
	else
		return RestorePolyArcFormat1(sw, bHole);
}


bool CGLFFile::GetNextPolygon(fstream& sw, bool* bHole)
{
	char cLine[512];
	bool bNotFound = true;

	while (sw.good())
	{
		sw.getline(cLine, 512);

		if (cLine[0] == '(' && cLine[2] == ')')
		{
			if (bHole != 0)
			{
				*bHole = cLine[1] == '-';
				return true;
			}
			else if (cLine[1] == '+')
			{
				return true;	
			}
		}
	}
	return false;
}





C2DPolyArc* CGLFFile::RestorePolyArcFormat2(fstream& sw, bool* bHole)
{
	if (!GetNextPolygon(sw, bHole))
		return 0;

	char cLine[512];

	C2DPolyArc* pResult = 0;

	double fx(0), fy(0), fr(0);
	unsigned char b1(0), b2(0);

	while (sw.good() && sw.getline(cLine, 512) && 
		sscanf_s(cLine, "%20le %20le %20le %1d %1d", &fx, &fy, &fr, &b1, &b2) >= 2)
	{
		if (pResult == 0)
		{
			pResult = new C2DPolyArc();
			pResult->SetStartPoint(C2DPoint(fx, fy));
		}
		else if (fr != 0)
		{
			pResult->LineTo(C2DPoint(fx, fy), fr, b1 == 1, b2 == 1);
			fr = 0;
		}
		else
		{
			pResult->LineTo(C2DPoint(fx, fy));
		}
	}

	if (pResult)
	{
		// end
		pResult->Close();
	}

	return pResult;
}


C2DPolyArc* CGLFFile::RestorePolyArcFormat1(fstream& sw, bool* bHole)
{
	if (!GetNextPolygon(sw, bHole))
		return 0;

	char cLine[512] = {0};

	C2DPolyArc* pResult = 0;

	double fx(0), fy(0), fr(0), fx0(0), fy0(0), fr0(0);
	unsigned char b1(0), b2(0), b10(0), b20(0);

	while (sw.good() && sw.getline(cLine, 512) && 
		sscanf_s(cLine, "%20le %20le %20le %1d %1d", &fx, &fy, &fr, &b1, &b2) >= 2)
	{
		if (pResult == 0)
		{
			pResult = new C2DPolyArc;
			pResult->SetStartPoint(C2DPoint(fx, fy));
		}
		else if (fr0 != 0)
		{
			pResult->LineTo(C2DPoint(fx, fy), fr0, b10 == 1, b20 == 1);
		}
		else
		{
			pResult->LineTo(C2DPoint(fx, fy));
		}
		// Copy the old over
		fx0 = fx;
		fy0 = fy;
		fr0 = fr;
		b10 = b1;
		b20 = b2;
		// Flag as invalid for next time.
		fr = 0;
	}

	if (pResult)
	{
		if (fr0 != 0)
		{
			pResult->Close(fr0, b10 == 1, b20 == 1);
		}
		else
		{
			pResult->Close();
		}
	}

	return pResult;
}



/**--------------------------------------------------------------------------<BR>
CGLFFile::RestorePolygon
\brief RestorePolygon.
<P>---------------------------------------------------------------------------*/
C2DPolygon* CGLFFile::RestorePolygon(fstream& sw, bool* bHole)
{
	if (!GetNextPolygon(sw, bHole))
		return 0;

	char cLine[512];

	double fx(0), fy(0);

	C2DPointSet pts;

	while (sw.good() && sw.getline(cLine, 512) && 
		sscanf_s(cLine, "%20le %20le", &fx, &fy) == 2)
	{
		pts.Add(new C2DPoint(fx, fy));
	}

	if (pts.size() > 0)
	{
		C2DPolygon* pResult = new C2DPolygon;

		pResult->Create(pts);

		return pResult;
	}

	return 0;
}


