/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file GLFFile.h
\brief Declaration file for the GLFFile Class.

Declaration file for GLFFile, a class for saving a loading from .gfl (GeoLib format)
file. The file is stored as follows

Format 1 DEFAULT

+									The next shape is a rim.
-									The next shape is a hole for the previous rim.
float float							A point.
float float float boolean boolean	A line to this point and an arc to the NEXT point
START AND END POINT NOT THE SAME

Format 2

+									The next shape is a rim.
-									The next shape is a hole for the previous rim.
float float							A point.
float float float boolean boolean	An arc FROM the PREVIOUS point to this.

START AND END POINT THE SAME




<P>---------------------------------------------------------------------------*/


#pragma once


#include <fstream>

class C2DHoledPolyBaseSet;
class C2DPolygon;
class C2DPolyBase;
class C2DHoledPolygon;
class C2DHoledPolyBase;
class C2DPolygonSet;
class C2DHoledPolygonSet;
class C2DPoint;
class C2DPolyArc;
class C2DBaseSet;
class C2DPolyArcSet;
class C2DHoledPolyArcSet;
class C2DHoledPolyArc;

class CProgressUpdater;

using namespace std;

#ifdef _VIEW_EXPORTING
	#define VIEW_DECLSPEC		__declspec(dllexport)
#else 
	#ifdef _STATIC
		#define VIEW_DECLSPEC
	#else
		#define VIEW_DECLSPEC		__declspec(dllimport)
	#endif
#endif



class VIEW_DECLSPEC CGLFFile
{
public:
	/// Saves the polygon
	static bool Save(const char* strFile, const C2DPolyBase& rPoly, bool bAppend = false);
	/// Saves the polygon
	static bool Save(const char* strFile, const C2DHoledPolyBase& rPoly, bool bAppend = false);
	/// Saves the polygons in the set.
	static bool Save(const char* strFile, const C2DBaseSet& rPoly , bool bAppend = false, CProgressUpdater* pUpdater = NULL);
	
	
	/// Loads the first polygon found
	static C2DPolygon* LoadPolygon(const char* strFile);
	/// Loads the first polygon found
	static C2DHoledPolygon* LoadHoledPolygon(const char* strFile);
	/// Loads the first polyarc found
	static C2DPolyArc* LoadPolyArc(const char* strFile);
	/// Loads the first polyarc found
	static C2DHoledPolyArc* LoadHoledPolyArc(const char* strFile);

	/// Loads into a set of polygons - arc ignored if present. Holes ignored.
	static bool Load(const char* strFile, C2DPolygonSet& polys, CProgressUpdater* pUpdater = NULL);
	/// Loads into a set of polygons - arc ignored if present
	static bool Load(const char* strFile, C2DHoledPolygonSet& polys, CProgressUpdater* pUpdater = NULL);
	/// Loads into a set of polyarcs. Holes ignored.
	static bool Load(const char* strFile, C2DPolyArcSet& polys, CProgressUpdater* pUpdater = NULL);
	/// Loads into a set of polyarcs.
	static bool Load(const char* strFile, C2DHoledPolyArcSet& polys, CProgressUpdater* pUpdater = NULL);
	
	/// Loads into a set. Arced polys created when necessary.
	static bool Load(const char* strFile, C2DBaseSet& polys, CProgressUpdater* pUpdater = NULL);

	static void SetFormat(bool bFormat2);

	static bool GetFormat(void);

private:

	static unsigned int GetFileLength(fstream& sw);

	static bool InitSave(fstream& sw, const char* strFile, bool bAppend = false);

	static bool InitLoad(fstream& sw, const char* strFile);



	static void Store(fstream& sw, const C2DPolyBase& pol);

	static void StoreFormat1(fstream& sw, const C2DPolyBase& pol);

	static void StoreFormat2(fstream& sw, const C2DPolyBase& pol);

	static bool Store(fstream& sw, const C2DHoledPolyBase& pol);


	static C2DPolygon* RestorePolygon(fstream& sw, bool* bHole = 0);

	static C2DPolyArc* RestorePolyArc(fstream& sw, bool* bHole = 0);

	static C2DPolyArc* RestorePolyArcFormat1(fstream& sw, bool* bHole = 0);

	static C2DPolyArc* RestorePolyArcFormat2(fstream& sw, bool* bHole = 0);

	// Moves to the next polygon
	static bool GetNextPolygon(fstream& sw, bool* pbHole = 0);

	static bool m_sbFormat2;
};
