/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file GeoDraw.h
\brief Declaration file for the CGeoDraw class. 

\class CGeoDraw
\brief Class which is used to draw GeoLib entities. 
<P>---------------------------------------------------------------------------*/

#ifndef _GEOLIB_GEODRAW_H
#define _GEOLIB_GEODRAW_H

class C2DLine;
class C2DPoint;
class C2DArc;
class C2DCircle;
class C2DRect;
class C2DPolyBase;
class C2DSegment;
class C2DTriangle;
class C2DHoledPolyBase;
class CDC;
class CPen;
class CBrush;

class CGeoDraw
{
public:
	/// Constructor
	CGeoDraw(void);
	/// Destructor
	~CGeoDraw(void);
	/// Sets the scale
	void SetScale(const C2DPoint& pt);
	/// Sets the offset
	void SetOffset(const C2DPoint& pt);

	/// Draws the shape
	void DrawPoints(const C2DPolyBase& Poly, CDC* pDC, unsigned long colorRef) const;
	/// Draws the shape
	void DrawPoints(const C2DHoledPolyBase& Poly, CDC* pDC, unsigned long colorRef) const;

	/// Draws the shape
	void Draw(const C2DLine& Line, CDC* pDC, CPen* pPen = 0) const;
	/// Draws the shape
	void Draw(const C2DArc& Arc, CDC* pDC, CPen* pPen = 0) const;
	/// Draws the shape
	void Draw(const C2DCircle& Circle, CDC* pDC, CPen* pPen = 0) const;
	/// Draws the shape
	void Draw(const C2DPoint& Pt, CDC* pDC, CPen* pPen = 0) const;
	/// Draws the shape
	void Draw(const C2DRect& Rect, CDC* pDC, CPen* pPen = 0) const;
	/// Draws the shape
	void Draw(const C2DPolyBase& Poly, CDC* pDC, CPen* pPen = 0) const;
	/// Draws the shape
	void Draw(const C2DSegment& Seg, CDC* pDC, CPen* pPen = 0) const;
	/// Draws the shape
	void Draw(const C2DTriangle& Triangle, CDC* pDC, CPen* pPen = 0) const;
	/// Draws the shape
	void Draw(const C2DHoledPolyBase& HoledPoly, CDC* pDC, CPen* pPen = 0) const;
	/// Draws the shape filled.
	void DrawFilled(const C2DPolyBase& Poly, CDC* pDC, CBrush* pBrush = 0, CPen* pPen = 0) const;
	/// Draws the shape filled.
	void DrawFilled(const C2DHoledPolyBase& HoledPoly, CDC* pDC, CBrush* pBrush = 0, CPen* pPen = 0) const;
	/// Draws the shape filled.
	void DrawFilled(const C2DCircle& Circle, CDC* pDC, CBrush* pBrush = 0, CPen* pPen = 0) const;
	/// Draws the shape filled.
	void DrawFilled(const C2DRect& Rect, CDC* pDC, CBrush* pBrush = 0, CPen* pPen = 0) const;
	/// Draws the shape filled.
	void DrawFilled(const C2DSegment& Seg, CDC* pDC, CBrush* pBrush = 0, CPen* pPen = 0) const;
	/// Draws the shape filled.
	void DrawFilled(const C2DTriangle& Triangle, CDC* pDC, CBrush* pBrush = 0, CPen* pPen = 0) const;
	/// Gets the scale
	const C2DPoint& GetScale(void) const;
	/// Gets the offset
	const C2DPoint& GetOffSet(void) const;

	void operator=(CGeoDraw& rOther);

private:
	void ScaleAndOffSet(C2DPoint& pt) const;
	C2DPoint* m_pOffset;
	C2DPoint* m_pScale;
};

#endif