#include "StdAfx.h"
#include "GeoView.h"
#include "ProgressUpdater.h"
#include "ShapeFile.h"
#include "GLFFile.h"
#include <math.h>



CGeoView::CGeoView(void)
{
	m_ScreenManager.SetYFlip(true);

	m_pBitMap = NULL;

	m_pDC = NULL;

	m_BackGround = RGB(170, 170, 170);

	m_Boundaries = RGB(170, 170, 170);
	
	m_Shapes = RGB(25, 25, 25);

	m_bPointsOnly = false;
}

void CGeoView::operator=(CGeoView& rOther)
{
	this->ClearCanvas();

	m_BackGround = rOther.m_BackGround;
	m_Boundaries = rOther.m_Boundaries;
	m_Drawer = rOther.m_Drawer;
	m_ScreenManager = rOther.m_ScreenManager;
	m_Shapes = rOther.m_Shapes;
	m_bPointsOnly = rOther.m_bPointsOnly;

	int nWidth = m_ScreenManager.GetWindowsRect().Width();
	int nHeight = m_ScreenManager.GetWindowsRect().Height();

	if (rOther.m_pBitMap)
	{
		m_pBitMap = new CBitmap();
		m_pBitMap->CreateCompatibleBitmap(rOther.m_pDC, nWidth, nHeight);
	}
	if (rOther.m_pDC)
	{
		this->m_pDC = new CDC();
		m_pDC->CreateCompatibleDC(rOther.m_pDC);
	}

	m_pDC->SelectObject(m_pBitMap);

	m_pDC->BitBlt(0, 0, nWidth, nHeight, rOther.m_pDC, 0, 0, SRCCOPY);
}


CGeoView::~CGeoView(void)
{
	ClearCanvas();
}


CDC* CGeoView::GetDisplay(void)
{
	return m_pDC;
}


const CDC* CGeoView::GetDisplay(void) const
{
	return m_pDC;
}



void CGeoView::CreateDefaultView(const C2DHoledPolyBaseSet& polys)
{
	C2DRect RealRect;

	polys.GetBoundingRect(RealRect);

	m_ScreenManager.SetRealRect( RealRect );

}


void CGeoView::ClearCanvas(void)
{
	if (m_pBitMap)
	{
		delete m_pBitMap;
		m_pBitMap = NULL;
	}

	if (m_pDC)
	{
		delete m_pDC;
		m_pDC = NULL;
	}
}

bool CGeoView::IsInitialised(void) const
{
	return m_pDC != 0;
}


void CGeoView::Initialise(CDC* pDC, unsigned int nWidth, unsigned int nHeight)
{
	ClearCanvas();

	m_pBitMap = new CBitmap();

	m_pDC = new CDC();

	CRect rect(0, 0, nWidth, nHeight);
	
	m_ScreenManager.SetWindowsRect(rect);

	m_pDC->CreateCompatibleDC(pDC);

	m_pBitMap->CreateCompatibleBitmap(pDC, nWidth, nHeight);

	m_pDC->SelectObject(m_pBitMap);

	m_pDC->FillSolidRect (0, 0, nWidth, nHeight, m_BackGround);
}



void CGeoView::Display(CDC* pDC, unsigned int nXOffset, unsigned int nYOffset)
{
	if (m_pDC && m_pBitMap)
	{
		unsigned int nWidth = m_ScreenManager.GetWindowsRect().Width();
		unsigned int nHeight = m_ScreenManager.GetWindowsRect().Height();

		pDC->BitBlt(nXOffset, nYOffset, nWidth, nHeight, m_pDC, 0, 0, SRCCOPY);

	//	pDC->TransparentBlt(nXOffset, nYOffset, nWidth, nHeight, m_pDC, 0, 0, 
	//									nWidth, nHeight, this->m_BackGround);
	}
}

void CGeoView::Display(CDC* pDC, const CRect& rect)
{
	if (m_pDC && m_pBitMap)
	{
		unsigned int nWidth = m_ScreenManager.GetWindowsRect().Width();
		unsigned int nHeight = m_ScreenManager.GetWindowsRect().Height();

		pDC->StretchBlt(rect.left, rect.top, rect.Width(), rect.Height(), 
						m_pDC, 0, 0, nWidth, nHeight, SRCCOPY);
	}
}


void CGeoView::Display(CDC* pDC, unsigned int nXOffset, unsigned int nYOffset, 
								unsigned int nWidth, unsigned int nHeight)
{
	if (m_pDC && m_pBitMap)
	{
		unsigned int nWidth2 = m_ScreenManager.GetWindowsRect().Width();
		unsigned int nHeight2 = m_ScreenManager.GetWindowsRect().Height();

		pDC->StretchBlt(nXOffset, nYOffset, nWidth, nHeight, 
						m_pDC, 0, 0, nWidth2, nHeight2, SRCCOPY);
	}
}


void CGeoView::Draw( const C2DHoledPolyBaseSet& polys)	
{
	if (m_pDC == NULL)
		return;

	unsigned int nWidth = m_ScreenManager.GetWindowsRect().Width();
	unsigned int nHeight = m_ScreenManager.GetWindowsRect().Height();	

	m_pDC->FillSolidRect(0, 0, nWidth, nHeight, m_BackGround);

	CPen BoundaryPen(PS_SOLID , 1, m_Boundaries);

	CBrush CountryBrush(m_Shapes);

	m_Drawer.SetScale(m_ScreenManager.GetScale());

	m_Drawer.SetOffset(m_ScreenManager.GetOffset());

	DrawPolygonSet(&polys, m_pDC, &CountryBrush, &BoundaryPen);
}



void CGeoView::DrawPolygonSet(const C2DHoledPolyBaseSet* pPolySet, CDC* pDC, CBrush* pBrush, CPen* pPen)
{
	for (unsigned int i = 0 ; i < pPolySet->size() ; i++)
	{
		DrawPolygon(*pPolySet->GetAt(i), pDC, pBrush, pPen);
	}
}


void CGeoView::DrawPolygon(const C2DHoledPolyBase& Poly, CDC* pDC, CBrush* pBrush, CPen* pPen)
{
	if (IsDrawable(*Poly.GetRim()))
	{
		if (m_bPointsOnly)
		{
			m_Drawer.DrawPoints(Poly, pDC, m_Boundaries);
		}
		else
		{
			m_Drawer.DrawFilled(Poly, pDC, pBrush, pPen);
		}
	}
}



bool CGeoView::IsDrawable(const C2DPolyBase& Poly) const
{
	C2DRect BR;
	Poly.GetBoundingRect(BR);
	C2DRect BRScaled(BR);
	BRScaled.Scale(this->m_ScreenManager.GetScale());

	double dArea = BRScaled.GetArea();

	dArea = fabs(dArea);

	if (dArea < 4.0)
		return false;

	if (m_ScreenManager.GetRealWindowsRect().Overlaps(BR))
		return true;

	return false;
}



CScreenManager& CGeoView::GetScreenManager()
{
	return this->m_ScreenManager;
}

void CGeoView::SetDrawPointsOnly(bool bPointsonly)
{
	m_bPointsOnly = bPointsonly;
}

void CGeoView::SetBackground(COLORREF rgb)
{
	m_BackGround = rgb;
}

void CGeoView::SetBoundaries(COLORREF rgb)
{
	m_Boundaries = rgb;
}


void CGeoView::SetShapes(COLORREF rgb)
{
	m_Shapes = rgb;
}

COLORREF CGeoView::GetBackground(void)
{
	return m_BackGround;
}

COLORREF CGeoView::GetBoundaries(void)
{
	return m_Boundaries;
}

COLORREF CGeoView::GetShapes(void)
{
	return m_Shapes;
}