/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file ScreenManager.h
\brief Declaration file for the CScreenManager Class.

Declaration file for CScreenManager, a class which can be used to help 
calculate drawing offsets and scales. 

\class CScreenManager
\brief CScreenManager, a class which can be used to help calculate drawing offsets 
and scales. 

CScreenManager, a class which can be used to help calculate drawing offsets 
and scales. 
<P>---------------------------------------------------------------------------*/

#ifndef _GEOLIB_CSCREENMANAGER_H
#define _GEOLIB_CSCREENMANAGER_H

#include <afxwin.h> // Drawing
#include "C2DRect.h"


#ifdef _VIEW_EXPORTING
	#define VIEW_DECLSPEC		__declspec(dllexport)
#else 
	#ifdef _STATIC
		#define VIEW_DECLSPEC
	#else
		#define VIEW_DECLSPEC		__declspec(dllimport)
	#endif
#endif

class VIEW_DECLSPEC CScreenManager
{
public:
	/// Constructor.
	CScreenManager(void);
	/// Destructor.
	~CScreenManager(void);

	/// Returns the offset
	const C2DPoint& GetOffset(void) const;
	/// Returns the scale
	const C2DPoint& GetScale(void) const;
	/// Returns the windows rect
	void SetWindowsRect(const CRect& WRect);
	/// Returns the real rect
	void SetRealRect(const C2DRect& RRect);
	/// Zoom
	void ZoomOut(double dFactor);
	/// Scrolls the real world rect by a real world amount
	void ScrollScreen(const C2DVector& Vector);
	/// Scrolls the real world rect by a screen based amount i.e. pixels.
	void ScrollReal(const C2DVector& Vector);
	/// Returns the windows rect
	const CRect& GetWindowsRect(void) const;
	/// Returns the real world rect
	const C2DRect& GetRealRect(void) const;
	/// Returns the real world rect representing the screen rect
	const C2DRect& GetRealWindowsRect(void) const;
	/// Returns the x flip flag
	bool GetXFlip(void) const {return m_bFlipX;}
	/// Returns the y flip flag
	bool GetYFlip(void) const {return m_bFlipY;}
	/// Sets the x flip flag
	void SetXFlip(bool bXFlip) {m_bFlipX = bXFlip;}
	/// Sets the y flip flag
	void SetYFlip(bool bYFlip) {m_bFlipY = bYFlip;}
	/// Returns a real world point from a point on the screen.
	C2DPoint MapScreenToReal(const CPoint& pt) const;
	/// Returns a point on the screen from a real world point.
	CPoint MapRealToScreen(const C2DPoint& pt) const;

	void operator=(CScreenManager& rOther);

private:

	/// Calculates the offset and scale.
	void Calculate(void);

	/// The windows rectangle that we are trying to project this onto
	CRect m_WindowsRect;
	/// The real world rectangle that we are viewing
	C2DRect m_RealRect;
	/// The resulting real world rectangle that the screen represents (note that this is the same as the above
	/// so long as the aspect ration is maintained).
	C2DRect m_RealWindowsRect;

	/// True if the Y dimension is to be flipped because e.g. because of Windows
	bool m_bFlipY;
	/// True if the Y dimension is to be flipped
	bool m_bFlipX;
	/// The offest
	C2DPoint m_ptOffset;
	/// The scale
	C2DPoint m_ptScale;

};


#endif