/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file ShapeFile.h
\brief Declaration file for the CShapeFile Class.

Declaration file for CShapeFile, a class for loading from and writing to ESRI 
ShapeFile.

\class CShapeFile.
\brief class for loading from and writing to ESRI ShapeFile.
<P>---------------------------------------------------------------------------*/


#pragma once


#include <vector>

using namespace std;

class CProgressUpdater;
class C2DHoledPolyBaseSet;
class C2DHoledPolyBase;
class C2DHoledPolygonSet;
class C2DHoledPolygon;
class CShapeFilePolygon;
class C2DPolygon;
class C2DPolyBase;
struct sSFPoint;


#ifdef _VIEW_EXPORTING
	#define VIEW_DECLSPEC		__declspec(dllexport)
#else 
	#ifdef _STATIC
		#define VIEW_DECLSPEC
	#else
		#define VIEW_DECLSPEC		__declspec(dllimport)
	#endif
#endif



class VIEW_DECLSPEC CShapeFile
{
public:
	/// Loads file creating an array of sets. Each polygon in an ESRI shapefile can
	/// be a set of polygons e.g. each set is a country with a mainland and possible
	/// islands.
	static bool Load(const char* strFileName, vector<C2DHoledPolygonSet*>& aPolygonSets, 
																CProgressUpdater* pUpdater = 0);
	/// Loads file into just one set. e.g. countries are thrown in together in one set.
	static bool Load(const char* strFileName, C2DHoledPolygonSet& aPolygons, 
																CProgressUpdater* pUpdater = 0);
	/// As above but with a base set. Straight line polygons created though.
	static bool Load(const char* strFileName, C2DHoledPolyBaseSet& aPolygons, 
																CProgressUpdater* pUpdater = 0);


	/// Saves the set.
	static bool Save(const char* strFileName, const C2DHoledPolygonSet& aPolygons, 
																CProgressUpdater* pUpdater = 0);
	/// Saves the sets grouped by set e.g. countries.
	static bool Save(const char* strFileName, const vector<C2DHoledPolygonSet*>& aPolygonSets, 
																CProgressUpdater* pUpdater = 0);
	/// Saves the set.
	static bool Save(const char* strFileName, const C2DHoledPolyBaseSet& aPolygonSets, 
																CProgressUpdater* pUpdater = 0);

private:

	static bool LoadPolygonFile(FILE* pFile, vector<C2DHoledPolygonSet*>& aPolygonSets, 
									long int nLength, CProgressUpdater* pUpdater = 0);

	static bool LoadPolygonFile(FILE* pFile, C2DHoledPolygonSet& aPolygons, 
									long int nLength, CProgressUpdater* pUpdater = 0);

	static bool LoadPolygonFile(FILE* pFile, C2DHoledPolyBaseSet& aPolygons, 
									long int nLength, CProgressUpdater* pUpdater = 0);

	static C2DHoledPolygonSet* CreatePolygonSet(const CShapeFilePolygon* pPoly);

	static C2DPolygon* CreatePolygon(const double* pPointList, unsigned int nPointCount);

	static bool OpenFile(FILE* pFile);

	static bool ReadPolygon(FILE* pFile, CShapeFilePolygon& pPoly);

	static void CreatePolygon(const C2DHoledPolyBase& rPoly, CShapeFilePolygon& pPoly);

	static void CreatePolygon(const C2DHoledPolygonSet& rPolys, CShapeFilePolygon& pPoly);

	static bool OpenSave(FILE* pFile);

	static void WritePoints(sSFPoint* pPoints, unsigned int& nCurrentPoint, const C2DPolyBase* pPoly, bool bClockwise);

	static bool Write(const CShapeFilePolygon* pPoly, unsigned int nIndex, FILE* pFile);

	static bool WriteBoundingBox(FILE* pFile, const C2DHoledPolygonSet& aPolygons);

	static bool WriteBoundingBox(FILE* pFile, const C2DHoledPolyBaseSet& aPolygons);

	static bool WriteBoundingBox(FILE* pFile, const vector<C2DHoledPolygonSet*>& aPolygons);

	static void WriteFileLength(FILE* pFile);

};
